@@ -30,7 +30,6 @@
 //!
 //! ```
 //! use std::collections::HashSet;
-//! # use std::convert::Infallible;
 //!
 //! use paxakos::{LogEntry, State};
 //! use uuid::Uuid;
@@ -175,6 +174,58 @@
 //! nodes which have joined the cluster or to catch up nodes that have fallen
 //! behind.
 //!
+//! ## Decorations
+//!
+//! Implementations of the [`Decoration`][crate::decoration::Decoration] trait
+//! can provide reusable functionality than _can_ but need not be used. Paxakos
+//! comes with several decorations (see below).
+//!
+//! ## Heartbeats (`heartbeats` flag)
+//!
+//! The heartbeats decoration sends a heartbeat message at regular intervals.
+//!
+//! ## Autofill (`autofill` flag)
+//!
+//! From time to time gaps will appear in the distributed log. For example due
+//! to [concurrency](#Concurrency) or dropped messages. When that happens, the
+//! `autofill` decoration will fill the gap, implicitly catching the node up or
+//! making sure that queued log entries may be applied.
+//!
+//! ## Track Leadership (`track-leadership` flag)
+//!
+//! The leadership tracking decoration infers which nodes are leading the
+//! cluster.
+//!
+//! ## Ensure Leadership (experimental, `ensure-leadership` flag)
+//!
+//! Similar to heartbeats, the ensure leadership decoration will append a log
+//! entry after none has been for a certain amount of time. However the goal of
+//! this decoration is to ensure there is a leader.
+//!
+//! ## Leases
+//!
+//! A cluster will often have shared resources which must be locked before they
+//! may be accessed. To account for node failures, locks time out unless they
+//! are refreshed. Such locks are commonly referred to as "leases".
+//!
+//! ### Leaser (experimental, `leaser` flag)
+//!
+//! `Leaser` makes taking, refreshing and releasing a lease as convenient as
+//! calling [`take_lease`][crate::leases::leaser::Leaser::take_lease]. The lease
+//! is refreshed as long as the returned value is held onto.
+//!
+//! ### Releaser (experimental, `releaser` flag)
+//!
+//! The releaser decoration makes sure that leases that have timed out are
+//! cleared away, releasing the underlying resource.
+//!
+//! ## Master Leases (experimental, `master-leases` flag)
+//!
+//! Master leases are a mechanism to allow passive local reads. A comprehensive
+//! description may be found in [Paxos Made Live][paxos-made-live].
+//!
+//! [paxos-made-live]: https://dl.acm.org/doi/10.1145/1281100.1281103
+//!
 //! # Protocol
 //!
 //! This section describes the Paxakos protocol. It is, for the most part, a
@@ -290,9 +341,9 @@
 //! [Passive mode][crate::node::Participation] is implemented and superficially
 //! tested. Thorough testing is still needed.
 //!
-//! ## 🌧️ Serialization 🌧️
+//! ## ⛅ Serialization ⛅
 //!
-//! Snapshot serialization needs to be reworked to allow for backward compat.
+//! Snapshot serialization is serde based and still maturing.
 //!
 //! ## ⛅ API Stability ⛅
 //!
@@ -334,6 +385,7 @@ pub mod append;
 pub mod applicable;
 #[cfg(feature = "autofill")]
 pub mod autofill;
+pub mod buffer;
 pub mod cluster;
 pub mod communicator;
 pub mod decoration;
@@ -345,7 +397,7 @@ pub mod invocation;
 pub mod leadership;
 #[cfg(feature = "leases")]
 pub mod leases;
-mod log;
+mod log_entry;
 pub mod node;
 #[cfg(feature = "prototyping")]
 pub mod prototyping;
@@ -371,18 +423,19 @@ pub use event::ShutdownEvent;
 #[doc(inline)]
 pub use invocation::Invocation;
 #[doc(inline)]
-pub use log::LogEntry;
+pub use log_entry::LogEntry;
 #[doc(inline)]
 pub use node::builder as node_builder;
 pub use node::Commit;
+pub use node::Core;
 pub use node::Node;
 pub use node::NodeBuilder;
 pub use node::NodeHandle;
 pub use node::NodeInfo;
-pub use node::NodeKernel;
 pub use node::NodeKit;
 pub use node::NodeStatus;
 pub use node::RequestHandler;
+pub use node::Shell;
 pub use node::Shutdown;
 #[doc(inline)]
 pub use state::State;
